/* eslint-disable no-dupe-keys */
import {all, call, put, takeLatest} from 'redux-saga/effects';
import SharePreference from '../../common/SharePreference';
import Config from '../../Config';
import * as actions from '../actions';
import {request} from '../api';
import {saga} from '../index';
import {watchAPIAnswer} from './upload.saga';

function* getAPIAuthen(action) {
  const {payload} = action;

  try {
    // yield put({type: actions.REQUEST_DATA_QUESTIONS_LOADING, payload: {}});
    const getDataAuthen = yield call(request, {
      url: `${Config.url}/api/auth`,
      method: 'post',
      body: payload.auth,
    });
    const {body} = getDataAuthen;
    SharePreference.token_hotqa = `Bearer ${body.access_token}`;

    yield put({
      type: actions.REQUEST_DATA_QUESTIONS,
      payload: {test_id: payload.test_id},
    });
  } catch (e) {
    yield put({
      type: actions.REQUEST_DATA_QUESTIONS_FAIL,
      payload: {message: e.message},
    });
  }
}

function* watchAPIAuthen() {
  yield takeLatest(actions.REQUEST_AUTHENCATION, getAPIAuthen);
}

function* getAPIQuestion(action) {
  const {payload} = action;

  try {
    // yield put({type: actions.REQUEST_DATA_QUESTIONS_LOADING, payload: {}});
    const getQuestions = yield call(request, {
      url: `${Config.url}/api/test/questions/${payload.test_id}`,
      method: 'get',
      headers: {Authorization: SharePreference.token_hotqa},
    });

    yield put({
      type: actions.REQUEST_DATA_QUESTIONS_SUCCESS,
      payload: {data: getQuestions.body.data},
    });
  } catch (e) {
    yield put({
      type: actions.REQUEST_DATA_QUESTIONS_FAIL,
      payload: {message: e.message},
    });
  }
}

function* watchAPIAccepted() {
  yield takeLatest(actions.REQUEST_DATA_QUESTIONS, getAPIQuestion);
}

function* uploadFile(action) {
  const {payload} = action;

  try {
    // yield put({type: actions.REQUEST_DATA_QUESTIONS_LOADING, payload: {}});
    const getUpload = yield call(request, {
      url: `${Config.url}/api/uploads`,
      method: 'post',
      headers: {Authorization: SharePreference.token_hotqa},
      body: payload,
    });
    payload.user_answer.value = getUpload.body.data;
    yield put({
      type: actions.REQUEST_UPLOAD_RECORD_SUCCESS,
      payload: {data: getUpload.body.data},

      type: actions.REQUEST_SUBMIT_USES_ANSWER,
      payload: {data: payload},
    });
  } catch (e) {
    yield put({
      type: actions.REQUEST_UPLOAD_RECORD_FAIL,
      payload: {message: e.message},
    });
  }
}

function* watchAPIUpload() {
  yield takeLatest(actions.REQUEST_UPLOAD_RECORD, uploadFile);
}

function* rootSaga() {
  yield all([
    watchAPIAuthen(),
    watchAPIAccepted(),
    watchAPIUpload(),
    watchAPIAnswer(),
  ]);
}

saga(rootSaga);
